/*
   Enero de 2009
   Creado por Oscar Rodrguez Parra ( oscarrp@gmail.com )
   http://www.sistemasorp.es
   Usa el cdigo libremente, pero menciona al autor del mismo.
*/
#include "main.h"
#include "lcd.c"
#include <ds1624.c>
#include "ds1307.c"

int8 tiempo[7]; // array con los valores de la fecha y hora
int8 indice = 0; // indice del array de fecha y hora
int8 modo = 1; // modo de funcionamiento (0 = configuracin, 1 = fecha y hora, 2 = temperatura, 3 = mensaje)
int8 contador = 0; // contador de tiempo en segundos

/*
Nombre:
   convertirDecimales
Utilidad:
   Transforma los decimales que nos devuelve el DS1624 (0 - 255) en valores decimales (0 - 99)
Parmetros:
   valor = valor del byte bajo de la temperatura
Devuelve:
   valor decimal entre 0 y 98 a intervalos de 6
*/
int convertirDecimales(int valor)
{
   switch(valor)
   {
      case 248: return 98; 
      case 240: return 94;
      case 232: return 90;
      case 224: return 87;
      case 216: return 83;
      case 208: return 80;
      case 200: return 77;
      case 192: return 75;
       case 184: return 72;
      case 176: return 70;
      case 168: return 68;
      case 160: return 66;
       case 152: return 62;
      case 144: return 60;
      case 136: return 58;
       case 128: return 55;
      case 120: return 53;
      case 112: return 50;
      case 104: return 48;
      case  96: return 44;
      case  88: return 40;
      case  80: return 37;
      case  72: return 34;
      case  64: return 30;
      case  56: return 28;
      case  48: return 23;
      case  40: return 20;
      case  32: return 17;
      case  24: return 13;
      case  16: return 10;
      case   8: return 05;
      default: return 00;
   }
}

/*
Nombre:
   leeCadenaEEPROM
Utilidad:
   Imprime en el LCD una cadena almacenada en la EEPROM hasta encontrar el caracter 0
Parmetros:
   address = direccin de la EEPROM donde empieza la cadena
*/
void leeCadenaEEPROM(int8 address)
{
   int8 caracter;
   
   caracter = read_eeprom(address);
   while(caracter != '\0')
   {
      lcd_putc(caracter);
      caracter = read_eeprom(++address);
   }
}

/*
Nombre:
   imprimeTiempo
Utilidad:
   Imprime la fecha y la hora en el LCD. Adems si esta en el modo de configuracin muestra el indice
*/
void imprimeTiempo()
{
   if(modo == 0)
   {
      printf(lcd_putc, "\f    %02d/%02d/%02d %d", tiempo[0], tiempo[1], tiempo[2], indice);
   }
   else
   {
      printf(lcd_putc, "\f    %02d/%02d/%02d", tiempo[0], tiempo[1], tiempo[2]);
   }
   printf(lcd_putc, "\n    %02d:%02d:%02d", tiempo[3], tiempo[4], tiempo[5]);
}

/*
Nombre:
   pulsacion
Utilidad:
   Funcin de la interrupcin de RB0 que o bien hace pasar al modo de configuracin o si se encuentra en el 
   aumenta los valores de la fecha u hora.
*/
#int_EXT
void pulsacion() 
{
   delay_ms(200);
   contador = 0;
   if(modo != 0)
   {
      modo = 0;
      indice = 0;
   }
   else if(contador < 2)
   {
      tiempo[indice]++;
      /* Hacemos que los valores esten entre los rangos permitidos */
      if(tiempo[0] > 31) tiempo[0] = 1;
      if(tiempo[1] > 12) tiempo[1] = 1;
      if(tiempo[3] > 23) tiempo[3] = 0;
      if(tiempo[4] > 59) tiempo[4] = 0;
      if(tiempo[5] > 59) tiempo[5] = 0;
   }
}

void main()
{

   int16 temp; // Variable donde se guarda la temperatura
   int16 max = 0; // Variable donde se guarda la temperatura mxima alcanzada
   int16 min = 0x6400; // Variable donde se guarda la temperatura minima alcanzada
   
   // Configuracin de los parmetros del PIC
   setup_timer_0(RTCC_INTERNAL|RTCC_DIV_1);
   setup_timer_1(T1_DISABLED);
   setup_timer_2(T2_DISABLED,0,1);
   setup_comparator(NC_NC_NC_NC);
   setup_vref(FALSE);
   setup_oscillator(OSC_4MHZ);
   enable_interrupts(int_ext);
   enable_interrupts(global);   
   
   init(); // Inicializamos el DS1624
   ds1307_init(); // Inicializamos el DS1307
   lcd_init(); // Inicializamos el LCD
   init_temp(); // Pedimos la primera medicin de temperatura para que d tiempo a leerla luego
  
   for(;;)
   {
      switch(modo)
      {
         case 0: // Modo de configuracin de fecha y hora
            if(contador == 2) // Si han pasado dos segundos pasamos al siguiente elemento de la fecha u hora
            {
               indice++;
               if(indice > 5) // Si ya se ha pasado por todos los elementos de fecha y hora, la establecemos.
               {
                  ds1307_set_date_time(tiempo[0],tiempo[1],tiempo[2], tiempo[6], tiempo[3],tiempo[4],tiempo[5]);            
                  modo = 1; // Pasamos al modo de presentacin de fecha y hora
               }
               contador = 0;
            }
            imprimeTiempo();
            break;
         case 1: // Modo de presentacin de fecha y hora
            ds1307_get_date(tiempo[0], tiempo[1], tiempo[2], tiempo[6]);
            ds1307_get_time(tiempo[3], tiempo[4], tiempo[5]);
            imprimeTiempo();
            if(contador == 10) // Si han pasado 10 segundos pasamos al modo de presentacin de temperatura
            {
               modo = 2;
               contador = 0;
            }
            break;
         case 2: // Modo de presentacin de temperatura
            temp = read_temp();
            init_temp();
            if(temp > max) // Si la temperatura es mayor que la mxima, la establecemos como mxima
            {
               max = temp;
            }
            if(temp < min) // Si la temperatura es menor que la minima, la establecemos como minima
            {
               min = temp;
            }
            // Si estamos en los 30 primeros segundos de media noche, restablecemos los valores minimo y mximo
            if(tiempo[3] == 0 && tiempo[4] == 0 && tiempo[5] < 30)
            {
               max = 0;
               min = 0x6400;
            }
            printf(lcd_putc, "\fTemp: %02d,%02d\xA1\nM%02d,%02d\xA1 m%02d,%02d\xA1",
               (int)(temp >> 8),convertirDecimales(temp & 0xFF),
               (int)(max >> 8),convertirDecimales(max & 0xFF),
               (int)(min >> 8),convertirDecimales(min & 0xFF)
               );
            if(contador == 10) // Si han pasado 10 segundos pasamos al modo de presentacin de mensaje
            {
               modo = 3;
               contador = 0;
            }
            break;  
         case 3: // Modo de presentacin de mensaje
            leeCadenaEEPROM(0);
            if(contador == 5) // Si han pasado 5 segundos pasamos al modo de presentacin de fecha y hora
            {
               modo = 1;
               contador = 0;
            }
            break;
      }
      contador++;
      delay_ms(1000);
   }
}


